# Generated by Django 3.0.6 on 2020-05-31 17:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('audit_trail', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entity',
            name='entity_name',
            field=models.CharField(max_length=64, unique=True, verbose_name='Entity Name'),
        ),
    ]
